Public Class NoncountingNumberException
    Inherits ApplicationException

    Private m_badNum As Decimal = New Decimal(0.0) ' COuld have used Double, too
    Private m_badNumIsValid As Boolean = False

    ' By using default values for parameters, this constructor
    ' can serve as the three most common exception constructors:
    ' Default, with a message, and message and inner exception
    Sub New(Optional ByVal msg As String = "", Optional ByVal innerExc As Exception = Nothing)
        MyBase.New(msg, innerExc)
    End Sub


    ' This constructor is used to initialize the badNum field
    ' so that people can figure out exactly what the number is.
    Sub New(ByVal offendingNum As Decimal, Optional ByVal msg As String = "", Optional ByVal innerExc As Exception = Nothing)
        MyBase.New(msg & offendingNum.ToString, innerExc)

        m_badNum = offendingNum
        m_badNumIsValid = True 'Remember that we now have a valid number
    End Sub

    Public ReadOnly Property hasBadNumber() As Boolean
        Get
            Return m_badNumIsValid
        End Get
    End Property

    Public Property badNumber() As Decimal
        Get
            If (m_badNumIsValid) Then
                Return m_badNum
            Else
                ' Note how we throw another exception, since this should never happen
                ' Also, we save the current exception (using the "Me" reference - same as "this" in C++/C#)
                ' for later use.
                Throw New ApplicationException("Not allowed to get a number if there isn't a number already", Me)
            End If
        End Get
        Set(ByVal Value As Decimal)
            m_badNumIsValid = True
            m_badNum = Value
        End Set
    End Property
End Class
